
#include <cstdlib>
#include <memory>

#include <spdlog/spdlog.h>

#include <fggl/app.hpp>
#include <fggl/ecs3/types.hpp>
#include <fggl/ecs3/module/module.h>
#include <fggl/util/service.h>

namespace fggl {

    App::App(const Identifer& name) : App::App( name, name ) {
    }

    App::App(const Identifer& name, const Identifer& folder ) :
        m_running(true),
        m_types(std::make_unique<ecs3::TypeRegistry>()),
        m_modules(std::make_unique<ecs3::ModuleManager>(*m_types)),
        m_states() {}

    int App::run(int argc, const char** argv) {

        {
            // activate the first state
            auto& state = m_states.active();
            state.activate();
        }

        while ( m_running ) {
            auto& state = m_states.active();

            m_modules->onUpdate();
            state.update();

            // window rendering to frame buffer
            if ( m_window != nullptr ) {
                m_modules->onFrameStart();
                m_window->frameStart();

                // get draw instructions
                fggl::gfx::Paint paint;
                state.render(paint);

                // execute draw instructions
                auto& graphics = m_window->graphics();
                graphics.draw2D( paint );

                m_window->frameEnd();
                m_modules->onFrameEnd();

				m_running = m_running && !m_window->wantClose();
            }
        }

        {
            // shutdown last state
            auto& state = m_states.active();
            state.deactivate();
        }

        return EXIT_SUCCESS;
    }

} //namespace fggl

